     
##################################################################################
#   Function Name     z.stat                                                     #
#   Purpose           calculates z statistic for parameters n1, n2, s1, s2       #
#                                          based on Schlag 2009                  #
#                                                                                #
#   Input variables:                                                             #
#                                     n1      length of first sample             #
#                                     n2      length of second sample            #
#                                     s1      number of successes, first sample  # 
#                                     s2      number of successes, second sample #
#   Return Values:                    number z statistic                         #
##################################################################################
#   Function Name     b.stat                                                     #
#   Purpose           calculates Boschloo statistic                              #
#                                                                                #
#   Input variables:                                                             #
#                                     n1      length of first sample             #
#                                     n2      length of second sample            #
#                                     s1      number of successes, first sample  #
#                                     s2      number of successes, second sample #
#   Return Values:                    number Boschloo statistic                  #
##################################################################################
#   Function Name     pr.reject                                                  #
#   Purpose           Calculates rejection probability H0:Pr(Y1 =1)>=Pr(Y2 =1)   #
#                                                                                #
#   Input variables:                                                             #
#                                     n1      length of first sample             #
#                                     n2      length of second sample            #
#                                     s1      number of successes, first sample  #
#                                     s2      number of successes, second sample #
#                                     stat    cut off                            #
#                                     type    ("z") type of test, "z"- z test,   #
#                                                           "b"- Boschloo test   #
#   Return Values:                                                               #
#                                     number  rejection probability              #
#   Uses:                             z.stat  if type="z"                        #
#                                     b.stat  if type="b"                        #  
##################################################################################
#   Function Name     cutoff                                                     #
#   Purpose           Finds cut off value.                                       #
#                                                                                #
#   Input variables:                                       ()                    #
#                                     n1      length of first sample             #
#                                     n2      length of second sample            #
#                                     alpha   (0.05) significance level of test  #
#                                     pl      (FALSE) should a graph be drawn    #
#                                     type    ("z") type of test, "z"- z test,   #
#                                                           "b"- Boschloo test   #
#   Return Values:                                                               #
#                                     number  cut off value                      #
#                                     graph   probability of rejection           #
#   Uses:                             pr.reject    rejection probability         #
##################################################################################
#   Function Name     type2                                                      #
#   Purpose           Calculates and plots type II error                         #
#                                                                                #
#   Input variables:                                                             #
#                                     n1      first sample size                  #
#                                     n2      second sample seze                 #
#                                     df      difference Pr(Y1 =1)+df<=Pr(Y2 =1) #
#                                     alpha   (0.05) significance level of test  #
#                                     type    ("z") type of test, "z"- z test,   #
#                                             "b"- Boschloo test                 #
#                                             "both"- Z and Boschloo test        #
#                                     plot    (FALSE) should a graph be drawn    #
#   Return Values:                                                               #
#                                     text    sample sizes, difference,          #
#                                             propability of type II error       #
#                                     graph   type II error                      #
#   Uses:                             pr.reject    rejection probability         #
#                                     cutoff   cut off value                     #
##################################################################################
#                                                                                #
#   Authors           Simina Jokubauskaite                                       #
#                     Tautvilas Janusauskas                                      #
#   Date              05.12.2011                                                 #
#   Version           1.0                                                        #
# For documentation see                                                          #
#       (Schlag, Karl H. 2009, Distribution-Free Tests - Suplementary Material)  #
#                                                                                #
#                                                                                #
##################################################################################


z.stat <-function(n1,n2,s1,s2)
  {
# calculates z statistic for parameters n1, n2, s1, s2,
      
      t1 <- ((s2/n2)-(s1/n1))
      t2 <- sqrt( ((s1+s2)/(n1+n2))*(1-((s1+s2)/(n1+n2)))*((1/n1)+(1/n2))   )  
      
      t1/t2
  }

b.stat <- function(n1,n2,s1,s2)
  {
# calculates p value from the Fisher exact test for paramters n1, n2, s1, s2

      k <- c(s2:min(n2,s1+s2))
      res <- sum((choose(n1,(s1+s2-k))*choose(n2,k))/(choose((n1+n2),(s1+s2))))
      res
  }


pr.reject <- function(n1,n2,y1,y2,stat,type="z")
{
# calculates rejection probability of test "type" given success probabilities y1 and y2
# this is only needed as subroutine for other functions

    sum1 <- 0
    sum2 <- 0

    for(s1 in 0:n1)
      {
          s2 <- 1:(n2-1)
          t1 <- choose(n1,s1) 
          t2 <- y1^s1
          t3 <- (1-y1)^(n1-s1)
          t4 <- choose(n2,s2)
          t5 <- y2^s2
          t6 <- (1-y2)^(n2-s2)
          t7 <- switch(type,z=as.numeric(z.stat(n1,n2,s1,s2) >= stat),b=as.numeric(sapply(s2, function(x)b.stat(n1, n2, s1, x))<=stat))
          sum1 <- sum1+sum(t1*t2*t3*t4*t5*t6*t7)
          
          if(s1<=(n1-1))
            {
                t77 <-  switch(type,z=as.numeric(z.stat(n1,n2,s1,n2) >= stat),b= as.numeric(b.stat(n1,n2,s1,n2) <= stat))
                t55 <-y2^n2
                sum2 <-sum2+sum(t1*t2*t3*t55*t77)
            }
      }
    
    ssum <- sum1+sum2
    ssum
 }


cutoff <- function(n1,n2,alpha=.05,type="z")
{
# computes the cutoff or critical value of the test of type "type"

    cutoffv <- switch(type,b=c(0:10000)/10000,z=c(1:4000)/1000)
    dff <- round(length(cutoffv)/2)
    x <- matrix((0:100)/100,nrow=1)

    while(dff>1)
      {
          if(switch(type,z=max(apply(x,2,function(x)pr.reject(n1=n1,n2=n2,x[1],x[1],stat=(cutoffv[dff]),type)))<=alpha,b=max(apply(x,2,function(x)pr.reject(n1=n1,n2=n2,x[1],x[1],stat=(cutoffv[dff]),type)))>=alpha))
         {
              cutoffv <- cutoffv[1:dff]
            
          dff <- round(length(cutoffv)/2)
         }
          else
            {
             cutoffv <- cutoffv[dff:length(cutoffv)]
             dff <- round(length(cutoffv)/2)
          }
   }
       
     switch(type,z=cutoffv[2],b=cutoffv[1])


if (type=="z") {
print(paste("reject at alpha=",alpha,"if z statistic >= ",switch(type,z=cutoffv[2],b=cutoffv[1])))}
else print(paste("reject at alpha=",alpha,"if b statistic <= ",switch(type,z=cutoffv[2],b=cutoffv[1])))
}


cutofft <- function(n1,n2,alpha=.05,type="z")
{
# only used within type2 function

    cutoffv <- switch(type,b=c(0:10000)/10000,z=c(1:4000)/1000)
    dff <- round(length(cutoffv)/2)
    x <- matrix((0:100)/100,nrow=1)

    while(dff>1)
      {
          if(switch(type,z=max(apply(x,2,function(x)pr.reject(n1=n1,n2=n2,x[1],x[1],stat=(cutoffv[dff]),type)))<=alpha,b=max(apply(x,2,function(x)pr.reject(n1=n1,n2=n2,x[1],x[1],stat=(cutoffv[dff]),type)))>=alpha))
         {
              cutoffv <- cutoffv[1:dff]
            
          dff <- round(length(cutoffv)/2)
         }
          else
            {
             cutoffv <- cutoffv[dff:length(cutoffv)]
             dff <- round(length(cutoffv)/2)
          }
   }
       
     switch(type,z=cutoffv[2],b=cutoffv[1])

}


type2 <- function(n1,n2,alpha=0.05,df,type="z",plot=FALSE,...)
  {
      cutofft <- switch(type,z=cutofft(n1,n2,alpha,type),b=cutofft(n1,n2,alpha,type),both=c(cutofft(n1,n2,alpha,type="z"),cutofft(n1,n2,alpha,type="b")))
      y2 <- matrix(((df*1000):1000)/1000,nrow=1)
                       
      ans <- switch(type,z=1-apply(y2,2,function(x)pr.reject(n1=n1,n2=n2,x[1]-df,x[1],stat=cutofft,type)),b=1-apply(y2,2,function(x)pr.reject(n1=n1,n2=n2,x[1]-df,x[1],stat=cutofft,type)),both=list(z=1-apply(y2,2,function(x)pr.reject(n1=n1,n2=n2,x[1]-df,x[1],stat=cutofft[1],type="z")),b=1-apply(y2,2,function(x)pr.reject(n1=n1,n2=n2,x[1]-df,x[1],stat=cutofft[2],type="b"))))
      
      if(plot==TRUE)
        {
            if(type!="both"){plot(ans,...)}
                else
                  {
                 
                      plot(ans[[1]],col=2)
                      lines(ans[[2]],type = "p",col=3)
                      legend("bottom",c("Z-test","Boschloo"),col = c(2,3),pch=4)
                }
        }else{}
      
      print(paste("n1=",n1,", n2=",n2,", difference=",df,sep=""))
      switch(type,z=print("Z-test"),b=print("Boschloo test"),both=print("Z-test and Boschloo"))
      probi <- if(type=="both")
        {
            g <- sapply(ans,function(x)round(max(x),3))
      paste(g[1], g[2])}
      else{round(max(ans),3)}
      print(paste("Probability of type 2 error:",probi))
}

#examples

# to find the statistic of the two tests, run:
# z.stat(11,9,6,8)
# b.stat(7,15,1,7)

# to find the cutoffs, run:
# cutoff(11,9,0.05,"z")
# cutoff(7,15,0.01,"z")

# to find type 2 errors when Pr(Y1 =1)+df <= Pr(Y2 =1) and alpha==.05
# type2(7, 15, type="z", alpha=0.05, plot=TRUE,df=0.2)
# type2(7, 15, type="both", plot=TRUE,df=0.35)

# to find the p value related to you data, you need to do 2 steps
# (1)
# compare the type 2 errors of the two tests, eg look for the 
# for the value of df such that the lowest of the two type 2 errors
# equals 0.5. the corresponding test achieves the largest range of
# Pr(Y2=1)-Pr(Y1=1) where type 2 error is at most 0.5

# eg for n1=11 and n2=9 with some iterations one needs to 
# run type2(9, 11, type="both", alpha=0.05, plot=TRUE, df) for different values of df
# and finds df=0.4, with z test achieving 0.5 while Boschloo gets 0.521
# verify this by running type2(9, 11, type="both", alpha=0.05, plot=TRUE, df=0.4)

# as we have found that z test is preferred (based on the above criteria)
# we now need to take our observations, eg s1=6 and s2=8
# and compare test statistic z.stat(11,9,6,8)=1.667388
# to cutoff  cutoff(11,9,0.05,"z")=1.762 (reject at 0.05 if z statistic >= 1.762
# to find that we cannot reject the null at 5% as 1.667388<1.762

