

########################################
######### Z Test Functions #############
########################################

# see documentation in README file

##------------------------------------------------
## EXACT UNCONDITIONAL Z TEST
##------------------------------------------------


ztestCutoffData <- function(counts10, counts01)
  {
    res <- (counts10 - counts01)/sqrt(counts10 + counts01)
    return(res)
  }

ztestCutoff <- function(cutoff, number.obs, plotif = FALSE,
                        steps = 500, type2 = FALSE, diff = 0,
                        verbose = FALSE)
  {
    if(type2 == FALSE)
      {
        range <- matrix(seq(from = 0, to = 0.5, length.out = steps),
                        nrow = 1)
        res <- apply(range, 2, ztestProbability,
                     z = cutoff, vequalst = TRUE,
                     n = number.obs)
      }
    else
      {
        range <- matrix(seq(from = 0, to = (1 - diff)/2,
                            length.out = steps), nrow = 1)
        res <- 1 - apply(range, 2, ztestProbability, z = cutoff,
                         n = number.obs, vequalst = TRUE,
                         d = diff)
      }

    if(plotif == TRUE)
      {
        plot(range, res)
      }
    else
      {}
    if(verbose == TRUE)
      {
        return(list(y.values = res,
                    y.max = max(res)))
      }
    else
      {
        return(max(res))
      }
  }

ztestPvalue <- function(number.obs, counts10, counts01, plotif = TRUE,
                        steps = 500)
  {
    res <- ztestCutoff(cutoff = ztestCutoffData(counts10 = counts10,
                         counts01 = counts01),
                       number.obs = number.obs,
                       plotif = plotif, steps = steps)
    return(res)
  }


ztestCutoffAuto <- function(start, number.obs, alpha,
                            increment = 0.01, steps = 500)
  {
    cutoff <- start
    y.max <- ztestCutoff(cutoff = cutoff,
                         number.obs = number.obs,
                         plotif = FALSE, steps = steps)

    if (y.max <= alpha)
      {
        stop("Start value may be too large. Please try a lower one.")
      }

    while(y.max > alpha)
      {
        y.max <- ztestCutoff(cutoff = cutoff,
                             number.obs = number.obs,
                             plotif = FALSE, steps = steps)["y.max"]
        cutoff <- cutoff + increment
      }

    return(cutoff)
  }


### Combinatorics
## Does nothing but calcute the combinatorics in the summation
## formula below.
comb <- function(n, b, c)
  {
    res <- gamma(n+1)/(gamma(b+1)*gamma(c+1)*gamma(n-b-c+1))
    return(res)
  }



### Probability of rejection of the z-test
ztestProbability <- function(t, v, z, n, vequalst = FALSE, diff = 0)
  {
    ## t, v ... prob of observing (0,1) and (1,0), respectively
    ## diff ... difference of v and t, positive number. To calculate
    ## the typeII error, given (the assumed true H1) v = t + diff
    ## z ... cutoff value
    ## n ... total number of observations
    ## ci, bi ... counts of (0,1) and (1,0), respectively

    if(vequalst == TRUE)
      {
        v <- t + diff
      }

    u <- (1 - v - t)

    ci.sums <- NULL

    for(i in 1:n)
      {
        ## The two indizes:
        ## ci runs from zero to some lower bound
        ## bi from 1 to n
        ci <- 0:min(i - 1, n - i)
        bi <- rep(i, length(ci))

        ## Calculate the different terms of the summation
        t1 <- comb(n, bi, ci)
        t2 <- v^bi
        t3 <- t^ci
        t4 <- u^(n - bi - ci)
        t5 <- as.numeric(((bi - ci)/sqrt(bi + ci)) >= z)
        ci.sums[i] <- sum(t1 * t2 * t3 * t4 * t5)
      }
    res <- sum(ci.sums)
    return(res)
  }
